//
//  CCViewController.m
//  MyApp
//
//  Created by Joanna Carter on 12/07/12.
//  Copyright (c) 2012 Carter Conulting. All rights reserved.
//


#import "CCViewController.h"
#import "Person.h"


// class extension declared in the implementation file is private
// to this class
@interface CCViewController ()

// private person property is "strong" because we "own" it
@property (strong, nonatomic) Person *person;

// private textField property is "weak" because the form is its owner
// we are only taking a reference to it.
//
// The IBOutlet marker is so that the property is visible in the
// from the form designer.
@property (weak, nonatomic) IBOutlet UITextField *textField;

@property (weak, nonatomic) IBOutlet UIButton *btn;

// "event handler" for the button being tapped
//
// The IBAction return type is equivalent to "void"
// and ensures the method can be seen in the form designer.
- (IBAction)btnTap:(id)sender;

@property (weak, nonatomic) IBOutlet UILabel *label;

@end


@implementation CCViewController

// Synthesize the accessor and mutator methods
//
// The underline prefix version of the property name
// is the private field that holds the state.
@synthesize person = _person;

@synthesize label = _label;

@synthesize textField = _textField;

@synthesize btn = _btn;

// Overriding this method allows us to indicate which orientations of the device
// we want to allow.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // support portrait (button at bottom) and both landscape directions
  return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

// overriding this method because it is called once the form has been deserialised
// and all controls have been fully created
- (void)viewDidLoad
{
  [super viewDidLoad];
  
  // create an instance of out Person class
  self.person = [[Person alloc] init];
  
  // this line tells the Person object to notify this class when
  // the "name" property changes its value
  [self.person addObserver:self forKeyPath:@"name" options:NSKeyValueObservingOptionNew context:nil];
}

// overriding this method because it is called when the form is been unloaded from memory
- (void)viewDidUnload
{
  // stop the Person object from sending notifications when its "name" property changes its value
  [self.person removeObserver:self forKeyPath:@"name"];
  
  // release our references to the controls on the form
  [self setTextField:nil];
  
  [self setLabel:nil];
  
  [self setBtn:nil];
  
  [super viewDidUnload];
}

// This method is designed to respond when the state of any specified properties of
// registered objects change their state (in our case, the "name" of the Person)
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
  // check to see if we are interested in the property that has changed
  if (keyPath != @"name")
    return;

  // check to see if we are interested in the object that has changed
  if (object != self.person)
    return;
  
  // check to see if the "change" dictionary contains a key/value pair
  // which contains the new value for the property
  if ([change.allKeys containsObject:NSKeyValueChangeNewKey])
  {
    // get the new value for the property
    //
    // The "id" type is a general pointer to any object type
    id name = [change valueForKey:NSKeyValueChangeNewKey];
    
    // if the value of the "name" property is still nil,
    // the dictionary will contain NSNull, which cannot be converted to a string
    if (name == [NSNull null])
    {
      return;
    }

    // assign the non-null string to the label's text property
    self.label.text = name;
  }
}

- (IBAction)btnTap:(id)sender
{
  // set the "name" property of the Person object
  // this will trigger the "observer", which will call the above method
  self.person.name = self.textField.text;
  
  //telling the text field to resign from responding will hide the keyboard
  [self.textField resignFirstResponder];
}

@end











